-- OnLoad Function
function MonkeySpeed_OnLoad(self)
	-- register events
	self:RegisterEvent("VARIABLES_LOADED")
	self:RegisterEvent("UNIT_NAME_UPDATE")			-- this is the event I use to get per character config settings
	self:RegisterEvent("PLAYER_ENTERING_WORLD")		-- this event gives me a good character name in situations where "UNIT_NAME_UPDATE" doesn't even trigger
	self:RegisterEvent("UNIT_ENTERED_VEHICLE")		-- add support for vehicle speed
	self:RegisterEvent("UNIT_EXITED_VEHICLE")
	
	-- register chat slash commands
	-- this command toggles the percent display
	SlashCmdList["MONKEYSPEED_PERCENT"] = MonkeySpeed_TogglePercent
	SLASH_MONKEYSPEED_PERCENT1 = "/monkeyspeedpercent"
	SLASH_MONKEYSPEED_PERCENT2 = "/mspercent"
	
	-- this command toggles the coloured speed bar display
	SlashCmdList["MONKEYSPEED_BAR"] = MonkeySpeed_ToggleBar
	SLASH_MONKEYSPEED_BAR1 = "/monkeyspeedbar"
	SLASH_MONKEYSPEED_BAR2 = "/msbar"
	
	-- this command toggles the whole speed bar display
	SlashCmdList["MONKEYSPEED_DISPLAY"] = MonkeySpeed_ToggleDisplay
	SLASH_MONKEYSPEED_DISPLAY1 = "/monkeyspeed"
	SLASH_MONKEYSPEED_DISPLAY2 = "/mspeed"
	
	-- this command toggles the lock
	SlashCmdList["MONKEYSPEED_LOCK"] = MonkeySpeed_ToggleLock
	SLASH_MONKEYSPEED_LOCK1 = "/monkeyspeedlock"
	SLASH_MONKEYSPEED_LOCK2 = "/mslock"
	
	-- this command toggles absolute or horizontal speed calculation
	SlashCmdList["MONKEYSPEED_ABSOLUTE"] = MonkeySpeed_ToggleAbsolute
	SLASH_MONKEYSPEED_ABSOLUTE1 = "/monkeyspeedabsolute"
	SLASH_MONKEYSPEED_ABSOLUTE2 = "/msabsolute"
	
	MonkeySpeedFrame:SetBackdropBorderColor(1.0, 0.6901960784313725, 0.0, 1.0)
	
	MonkeySpeedOptions()
end

-- OnEvent Function
function MonkeySpeed_OnEvent(self, event, ...)
	local arg1 = ...
	if (event == "VARIABLES_LOADED") then
		-- this event gets called when the player enters the world
		-- Note: on initial login this event will not give a good player name
		
		MonkeySpeed.m_bVariablesLoaded = true
		
		-- double check that the mod isn't already loaded
		if (not MonkeySpeed.m_bLoaded) then
			
			MonkeySpeed.m_strPlayer = UnitName("player")
			
			-- if MonkeySpeed.m_strPlayer is "Unknown Entity" get out, need a real name
			if (MonkeySpeed.m_strPlayer ~= nil and MonkeySpeed.m_strPlayer ~= UNKNOWNOBJECT) then
				-- should have a valid player name here
				MonkeySpeed_Init()
			end
		end
		
		-- exit this event
		return
		
	end -- PLAYER_ENTERING_WORLD
	if (event == "UNIT_NAME_UPDATE") then
		-- this event gets called whenever a unit's name changes (supposedly)
		--  Note: Sometimes it gets called when unit's name gets set to
		--  "Unknown Entity"
		
		-- double check that we are getting the player's name update
		if (arg1 == "player" and not MonkeySpeed.m_bLoaded) then
			-- this is the first place I know that reliably gets the player name
			MonkeySpeed.m_strPlayer = UnitName("player")
			
			-- if MonkeySpeed.m_strPlayer is "Unknown Entity" get out, need a real name
			if (MonkeySpeed.m_strPlayer ~= nil and MonkeySpeed.m_strPlayer ~= UNKNOWNOBJECT) then
				-- should have a valid player name here
				MonkeySpeed_Init()
			end
		end
		
		-- exit this event
		return
		
	end -- UNIT_NAME_UPDATE
	if (event == "PLAYER_ENTERING_WORLD") then
		-- this event gets called when the player enters the world
		--  Note: on initial login this event will not give a good player name
		
		-- double check that the mod isn't already loaded
		if (not MonkeySpeed.m_bLoaded) then
			
			MonkeySpeed.m_strPlayer = UnitName("player")
			
			-- if MonkeySpeed.m_strPlayer is "Unknown Entity" get out, need a real name
			if (MonkeySpeed.m_strPlayer ~= nil and MonkeySpeed.m_strPlayer ~= UNKNOWNOBJECT) then
				-- should have a valid player name here
				MonkeySpeed_Init()
			end
		end
		
		-- exit this event
		return
		
	end -- PLAYER_ENTERING_WORLD
	if (event == "UNIT_ENTERED_VEHICLE") then
		if arg1 == "player" then
			MonkeySpeed.m_strUnit = "vehicle"
		end
	end -- UNIT_ENTERED_VEHICLE
	if (event == "UNIT_EXITED_VEHICLE") then
		if arg1 == "player" then
			MonkeySpeed.m_strUnit = "player"
		end
	end -- UNIT_EXITED_VEHICLE
end

-- OnUpdate Function (heavily based off code in Telo's Clock)
function MonkeySpeed_OnUpdate(self, elapsed)
	
	-- if the speedometer's not loaded yet, just exit
	if (not MonkeySpeed.m_bLoaded) then
		return
	end
	
	-- how long since the last update?
	MonkeySpeed.m_iDeltaTime = MonkeySpeed.m_iDeltaTime + elapsed
	if (MonkeySpeed.m_iDeltaTime >= .1) then
		local iSpeed = GetUnitSpeed(MonkeySpeed.m_strUnit)
		if iSpeed and iSpeed >= 0 then
			MonkeySpeed.m_iDeltaTime = 0.0
			if MonkeySpeedConfig[MonkeySpeed.m_strPlayer].m_bAbsoluteSpeed or not (IsSwimming() or IsFlying()) then
				MonkeySpeed.m_fSpeed = MonkeySpeed_Round(iSpeed/7 * 100)
			else
				MonkeySpeed.m_fSpeed = MonkeySpeed_Round((iSpeed/7 * 100) * math.cos(GetUnitPitch(MonkeySpeed.m_strUnit)))
			end
			if (MonkeySpeedConfig[MonkeySpeed.m_strPlayer].m_bDisplayPercent) then
				-- Set the text for the speedometer
				MonkeySpeedText:SetText(format("%d%%", MonkeySpeed.m_fSpeed))
			end
			if (MonkeySpeedConfig[MonkeySpeed.m_strPlayer].m_bDisplayBar) then
				-- Set the colour of the bar
				if (MonkeySpeed.m_fSpeed == 0.0) then
					MonkeySpeedBar:SetVertexColor(1, 0, 0)
				elseif (MonkeySpeed.m_fSpeed < 100.0) then
					MonkeySpeedBar:SetVertexColor(1, 0.25, 0)
				elseif (MonkeySpeed.m_fSpeed == 100.0) then
					MonkeySpeedBar:SetVertexColor(1, 0.5, 0)
				elseif ((MonkeySpeed.m_fSpeed > 100.0) and (MonkeySpeed.m_fSpeed < 140.0)) then
					MonkeySpeedBar:SetVertexColor(0, 1, 0)
				elseif ((MonkeySpeed.m_fSpeed >= 140.0) and (MonkeySpeed.m_fSpeed < 200.0)) then
					MonkeySpeedBar:SetVertexColor(1, 0, 1)
				elseif ((MonkeySpeed.m_fSpeed >= 200.0) and (MonkeySpeed.m_fSpeed < 550.0)) then
					MonkeySpeedBar:SetVertexColor(0.5, 0, 1)
				elseif (MonkeySpeed.m_fSpeed >= 550.0) then
					MonkeySpeedBar:SetVertexColor(0, 0, 1)
				end
			end
		else
			if (MonkeySpeedConfig[MonkeySpeed.m_strPlayer].m_bDisplayPercent) then
				-- Set the text for the speedometer
				MonkeySpeedText:SetText("???%")
			end
			if (MonkeySpeedConfig[MonkeySpeed.m_strPlayer].m_bDisplayBar) then
				-- Set the colour of the bar
				MonkeySpeedBar:SetVertexColor(0, 0, 0)
			end
		end
	end
end

-- when the mouse goes over the main frame, this gets called
function MonkeySpeed_OnEnter(self)
	-- put the tool tip in the default position
	GameTooltip_SetDefaultAnchor(GameTooltip, self)
	
	-- set the tool tip text
	GameTooltip:SetText(MONKEYSPEED_TITLE_VERSION, MONKEYLIB_TITLE_COLOUR.r, MONKEYLIB_TITLE_COLOUR.g, MONKEYLIB_TITLE_COLOUR.b, 1)
	GameTooltip:AddLine(MONKEYSPEED_DESCRIPTION, GRAY_FONT_COLOR.r, GRAY_FONT_COLOR.g, GRAY_FONT_COLOR.b, 1)
	GameTooltip:Show()
end

function MonkeySpeed_OnMouseDown(self, button)
	-- if not loaded yet then get out
	if (MonkeySpeed.m_bLoaded == false) then
		return
	end
	
	if (button == "LeftButton" and MonkeySpeedConfig[MonkeySpeed.m_strPlayer].m_bLocked == false) then
		MonkeySpeedFrame:StartMoving()
	end
	
	-- right button on the title or frame opens up the MonkeyBuddy, if it's there
	if (button == "RightButton") then
		if (MonkeyBuddyFrame ~= nil and MonkeySpeedConfig[MonkeySpeed.m_strPlayer].m_bAllowRightClick == true) then
			MonkeyBuddy_ToggleDisplay()
			
			-- make MonkeyBuddy show the MonkeySpeed config
			MonkeyBuddySpeedTab_OnClick()
		end
	end
end

function MonkeySpeed_OnMouseUp(self, button)
	-- if not loaded yet then get out
	if (MonkeySpeed.m_bLoaded == false) then
		return
	end
	
	if (button == "LeftButton") then
		MonkeySpeedFrame:StopMovingOrSizing()
	end
end

function MonkeySpeed_Round(x)
	if(x - floor(x) > 0.5) then
		x = x + 0.5
	end
	return floor(x)
end

function MonkeySpeedOptions()
	if (GetAddOnDependencies("MonkeyBuddy") == nil) then
		-- Create main frame for information text
		local MonkeySpeedOptions = CreateFrame("FRAME", "MonkeySpeedOptions")
		MonkeySpeedOptions.name = MONKEYSPEED_TITLE
		InterfaceOptions_AddCategory(MonkeySpeedOptions)
		
		function MonkeySpeedOptions.default()
			MonkeySpeed_ResetConfig()
		end
		
		local MonkeySpeedOptionsText1 = MonkeySpeedOptions:CreateFontString(nil, "ARTWORK")
		MonkeySpeedOptionsText1:SetFontObject(GameFontNormalLarge)
		MonkeySpeedOptionsText1:SetJustifyH("LEFT") 
		MonkeySpeedOptionsText1:SetJustifyV("TOP")
		MonkeySpeedOptionsText1:ClearAllPoints()
		MonkeySpeedOptionsText1:SetPoint("TOPLEFT", 16, -16)
		MonkeySpeedOptionsText1:SetText(MONKEYSPEED_TITLE_VERSION)
		
		local MonkeySpeedOptionsText2 = MonkeySpeedOptions:CreateFontString(nil, "ARTWORK")
		MonkeySpeedOptionsText2:SetFontObject(GameFontNormalSmall)
		MonkeySpeedOptionsText2:SetJustifyH("LEFT") 
		MonkeySpeedOptionsText2:SetJustifyV("TOP")
		MonkeySpeedOptionsText2:SetTextColor(1, 1, 1)
		MonkeySpeedOptionsText2:ClearAllPoints()
		MonkeySpeedOptionsText2:SetPoint("TOPLEFT", MonkeySpeedOptionsText1, "BOTTOMLEFT", 8, -16)
		MonkeySpeedOptionsText2:SetWidth(340)
		MonkeySpeedOptionsText2:SetText(MONKEYSPEED_OPTIONS1)
		
		local MonkeySpeedOptionsText3 = MonkeySpeedOptions:CreateFontString(nil, "ARTWORK")
		MonkeySpeedOptionsText3:SetFontObject(GameFontNormalLarge)
		MonkeySpeedOptionsText3:SetJustifyH("LEFT") 
		MonkeySpeedOptionsText3:SetJustifyV("TOP")
		MonkeySpeedOptionsText3:SetTextColor(1, 0.65, 0)
		MonkeySpeedOptionsText3:ClearAllPoints()
		MonkeySpeedOptionsText3:SetPoint("TOPLEFT", MonkeySpeedOptionsText2, "BOTTOMLEFT", 0, -16)
		MonkeySpeedOptionsText3:SetWidth(340)
		MonkeySpeedOptionsText3:SetText(MONKEYSPEED_OPTIONS2)
	end
end
